from airflow.providers.google.cloud.operators.dataflow import DataflowTemplatedJobStartOperator
from ..general import conf, conn_account, bucket_raw, bucket_trn, bucket_des
from airflow.hooks.base_hook import BaseHook
import datetime
import json
import gcsfs
import ast

def subdag_trn(parent_dag_name, child_dag_name, json_gs,**kwargs):
    connection_airflow_yas_sa_sii_de = BaseHook.get_connection(conn_account)
    service_account_yas_sa_sii_de = ast.literal_eval(connection_airflow_yas_sa_sii_de.extra_dejson["keyfile_dict"])

    # Datos Generales para la ejecucion
    project         = conf["proy_sii"]
    snetwork        = conf["snetwork"]
    service_account_email = conf['service_account_sii']
    machine_type    = conf['machine_type']
    with gcsfs.GCSFileSystem(project=project, token=service_account_yas_sa_sii_de).open(json_gs) as f:
        jd = json.load(f)
    # Variables para ejecucion desde JSON
    url_raw = f"gs://{bucket_raw}/{jd['app']}/raw_{jd['table_name']}/"

    # Datos de TRN
    job_name_trn  = jd['job_name']
    url_trn       =  f"gs://{bucket_trn}/{jd['app']}/trn_{jd['table_name']}/"
    file_name_trn = f"trn_{jd['table_name']}"
    trn_template  = f"gs://{bucket_des}/{jd['app']}/TRN/tpl/{jd['trn_template_name']}" 
    
    folders = gcsfs.GCSFileSystem(project=project, token=service_account_yas_sa_sii_de).ls(url_raw)
    folders.sort()
    folder = folders[-1]
    
    date_folder = folder.split('/')[3]
    if len(date_folder)==10:
        url_source = f"gs://{folder}/raw_{jd['table_name']}-00000-of-00001.parquet"
        url_dest   = url_trn + date_folder+'/'+file_name_trn
        parent_dag_name_for_id = parent_dag_name.lower()
        
        print('url_source: ' + url_source)
        print('url_dest: ' + url_dest)
        
        dataflow = DataflowTemplatedJobStartOperator(
        task_id=f'{parent_dag_name_for_id}-{child_dag_name}',
        job_name=f'{job_name_trn}-{date_folder}',
        template=trn_template,
        location="us-east1",
        gcp_conn_id=conn_account,
                parameters={
                    'url_raw' : url_source,
                    'url_trn' : url_dest,
                    'folder_date': str(datetime.date.today()),
                },
        options={
            "subnetwork":snetwork,
            "serviceAccountEmail":service_account_email,
        },
        dataflow_default_options={
                'project'      : project,
                'zone'         : conf['zone'],
                'tempLocation' : f"gs://{conf['bucket_tmp']}/CTA/",
                'machineType'  : machine_type,
                'subnetwork'   : snetwork,  
                'serviceAccountEmail': service_account_email,
                'ipConfiguration': 'WORKER_IP_PRIVATE',
            },
            # dag=dag,
        )
        dataflow.execute(context=kwargs)


import json
from datetime import timedelta

#raw
archivo_config = '/home/airflow/gcs/data/yas-sii/conf.json'

#lee variables para el ambiente

def recupera_be_conf(archivo_conf=archivo_config):
    """
    Lee la configuracion desde un archivo de propiedades
    :return: json de configuracion
    """
    print(archivo_conf)
    with open(archivo_conf) as f:
        be_conf = json.load(f)
    
    print(be_conf)
    return be_conf

conf=recupera_be_conf(archivo_config)

project_id = conf["proy_sii"]
gce_region = conf["region"]
gce_zone = conf["zone"]
machineType_exe = conf["machine_type"]
snetwork = conf["snetwork"]
bucket_des = conf["bucket_des"]
bucket_raw = conf['bucket_raw']
bucket_trn =  conf["bucket_trn"]
# bucket_des = f"gs://{conf['bucket_des']}"
bucket_temp_path = f"gs://{conf['bucket_tmp']}/"
conn_account = conf["conn_account_sii"]
service_account = conf["service_account_sii"]

args = {
    'owner': 'sii-dwh',
    'depends_on_past': False,
    'email': service_account,
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 0,
    'retry_delay': timedelta(minutes = 5),
    'catchup':False,
    'provide_context': True,
    'dataflow_default_options': {
        'project' : project_id,
        'zone' : gce_zone,
        'tempLocation' : bucket_temp_path,
        'machineType' : machineType_exe,
        'serviceAccountEmail': service_account,
        'subnetwork' : snetwork 
    }
}

# trn

#lee variables para el ambiente
# bucket_des = conf["bucket_des"]

# bucket_temp_path=f"gs://{conf['bucket_tmp']}/CTA/"
# project_id=conf["proy_sii"]
# gce_region=conf["region"]
# gce_zone=["zone"]
# machineType_exe = conf["machine_type"]
# conn_account = conf["conn_account_sii"]
# service_account = conf["service_account_sii"]
# snetwork = conf["snetwork"]



#wrk

#lee variables para el ambiente
# bucket_des = conf["bucket_des"]
# bucket_temp_path="gs://{}/CTA/".format(conf['bucket_tmp'])
# project_id=conf["proy_sii"]
# gce_region=conf["region"]
# gce_zone=conf["zone"]
# machineType_exe = conf["machine_type"]
# conn_account = conf["conn_account_sii"]
# service_account = conf["service_account_sii"]
# snetwork = conf["snetwork"]



from airflow.providers.google.cloud.operators.dataflow import DataflowTemplatedJobStartOperator
from airflow.hooks.base_hook   import BaseHook
from ..general import conf, conn_account, project_id, snetwork
import datetime
import pytz
import json
import ast
import gcsfs

def subdag_raw(parent_dag_name, child_dag_name, args, json_gs,**kwargs):    
    # Getting xcom
    dia_op = kwargs['ti'].xcom_pull(task_ids='get-parameters', key='DIA_OPERACION')
    print(f"Xcom Value get-parameters:{dia_op}")
    dia_op_f = kwargs['ti'].xcom_pull(task_ids='get-parameters', key='DIA_OPERACION_F')
    print(f"Xcom Value get-parameters:{dia_op_f}")
    # Datos Generales para la ejecucion
    project = conf["proy_sii"]
    region = conf['region']
    machine_type = conf['machine_type']  
    bucket_des = f"gs://{conf['bucket_des']}"    
    temp_location = conf['bucket_tmp']
    service_account_email = conf['service_account_sii']
    connection_airflow_yas_sa_sii_de2 = BaseHook.get_connection(conn_account)
    service_account_yas_sa_sii_de2    = ast.literal_eval(connection_airflow_yas_sa_sii_de2.extra_dejson["keyfile_dict"])        
    with gcsfs.GCSFileSystem(project=project, token=service_account_yas_sa_sii_de2).open(json_gs) as f2:
        jd = json.load(f2)
        app= jd['app']
    max_workers  = jd['max_num_workers']
    num_workers  = jd['num_workers']
    app          = jd['app']
    job_name_raw = jd['job_name']
    target_table = jd['target_table_name']
    edit_date    = jd['st_modified']
    create_date  = jd['st_created']

    conf_bd= f"{bucket_des}/SII/config/conf_BD.json"
    print( f"bdconf - {conf_bd}") 

    try:
        if "mode" in jd.keys() and jd['mode'] == 'MBT':
            mode_dir= 'MBT'
        else:
            mode_dir= jd['app']
    except ValueError:
        mode_dir=  jd['app']
    
    raw_location = f"gs://{conf['bucket_raw']}/{mode_dir}"
    sql_file     = f"gs://{conf['bucket_des']}/{mode_dir}/RAW/sql_query/sql-query-{app.lower()}-{target_table}-00000-of-00001.txt"
    raw_template = f"gs://{conf['bucket_des']}/{mode_dir}/RAW/tpl/{jd['raw_template_name']}"
    
    date_execute = str(datetime.date.today())
    output_raw   = f"{raw_location}/{target_table}/{date_execute}/{target_table}"

    with gcsfs.GCSFileSystem(project=project_id, token=service_account_yas_sa_sii_de2).open(sql_file) as f:
        sql_string = f.read().decode('utf-8')
    parent_dag_name_for_id = parent_dag_name.lower()
    empty = """ """
    sql_where5 = empty   
    if edit_date  == "NA" :
        sql_where1 = empty
        sql_where2 = empty
        sql_where3 = empty
        sql_where4 = empty
        if  "Add_conditions" in jd.keys():
            sql_where5 = f""" WHERE ({jd['Add_conditions']})"""
    else:
        sql_where1 = f""" WHERE (CAST("{create_date}" AS DATE) >= CAST('"""
        sql_where2 = f"""' AS DATE) AND CAST("{create_date}" AS DATE) <= CAST('"""
        sql_where3 = f"""' AS DATE) )  OR  (CAST("{edit_date}" AS DATE) >= CAST('"""
        sql_where4 = f"""' AS DATE)  AND CAST("{edit_date}" AS DATE) <= CAST('"""
        if  "Add_conditions" in jd.keys():
            sql_where5 = f"""' AS DATE)) AND ({jd['Add_conditions']})"""
        else:
            sql_where5 = """' AS DATE)) """


    dataflow = DataflowTemplatedJobStartOperator(
        template=raw_template,
        job_name=f'{job_name_raw}-{date_execute}',
        task_id =f'{parent_dag_name_for_id}-{child_dag_name}', 
        location=region,
        parameters={
            'sql_query' : sql_string + sql_where1 + dia_op + sql_where2 + dia_op_f + sql_where3 + dia_op + sql_where4 + dia_op_f + sql_where5,
            'output_raw': output_raw,
        },
        default_args=args,
        options={
            'machineType' : machine_type,
            'maxWorkers'  : max_workers,
            'numWorkers'  : num_workers,
        },
        dataflow_default_options={
            'project' : project,
            'zone'    : conf['zone'],
            'tempLocation': f"gs://{conf['bucket_tmp']}/CTA/",
            'machineType' : machine_type,
            'maxWorkers'  : max_workers,
            'numWorkers'  : num_workers,
            'subnetwork'  : snetwork,  
            'serviceAccountEmail': service_account_email,
            'ipConfiguration': 'WORKER_IP_PRIVATE',
        },
        gcp_conn_id=conn_account,
        # dag=dag,
        )
    dataflow.execute(context=kwargs)

def get_config(**kwargs):
    import time
    # {"extraction":"I","delta":90}   incremental, desde hace 90 dias
    # {"extraction":"F"}              full - todo desde "2010-01-01"
    # {"extraction":"R","exec_date":"2022-04-19","exec_date2":"2022-04-21"} #Rango de fecha inicio y fin
    print("---------------- inicio de config")
    CST = pytz.timezone("America/Mexico_City")
    DATE_FORMAT = "%Y-%m-%d"
    dia_op      = datetime.datetime.now(tz=CST)  -datetime.timedelta(days=5)
    dia_op2     = datetime.datetime.now(tz=CST) 
    
    EXTRACTION_P    = "I"            #Default
    DELTA_DIAS      = 5
    DIA_OPERACION   = dia_op.strftime(DATE_FORMAT)
    DIA_OPERACION_F = dia_op2.strftime(DATE_FORMAT)
    
    try:
        if kwargs['params'] and "extraction" in kwargs['params'].keys():
            print(f"---------------- kwargs - {kwargs['params']}")
            EXTRACTION_P = kwargs['params']["extraction"] 
            if   EXTRACTION_P == "I":
                DELTA_DIAS     = kwargs['params']["delta"]
                dia_op         = datetime.datetime.now(tz=CST)  -datetime.timedelta(days=DELTA_DIAS)
                DIA_OPERACION  = dia_op.strftime (DATE_FORMAT)
                DIA_OPERACION_F= dia_op2.strftime(DATE_FORMAT)
            elif EXTRACTION_P == "F":
                DIA_OPERACION  = datetime.datetime.strptime("2010-01-01" ,DATE_FORMAT)
                DIA_OPERACION_F= dia_op2.strftime(DATE_FORMAT)
            elif EXTRACTION_P == "R":
                DIA_OPERACION  = datetime.datetime.strptime(kwargs['params']["exec_date"] ,DATE_FORMAT)
                DIA_OPERACION_F= datetime.datetime.strptime(kwargs['params']["exec_date2"],DATE_FORMAT)
                DIA_OPERACION  = DIA_OPERACION.strftime (DATE_FORMAT)
                DIA_OPERACION_F= DIA_OPERACION_F.strftime(DATE_FORMAT)
        elif """{{ ti.xcom_pull(task_ids='get-parameters', key='EXTRACTION_P')  }}""" in ('F','I','R'):
            print(f"---------------- xcom_pull {kwargs['EXTRACTION_P']}")
            EXTRACTION_P   = """{{ ti.xcom_pull(task_ids='get-parameters', key='EXTRACTION_P')  }}"""
            DELTA_DIAS     = """{{ ti.xcom_pull(task_ids='get-parameters', key='DELTA_DIAS'     }}"""
            DIA_OPERACION  = """{{ ti.xcom_pull(task_ids='get-parameters', key='DIA_OPERACION') }}"""
            DIA_OPERACION_F= """{{ ti.xcom_pull(task_ids='get-parameters', key='DIA_OPERACION_F') }}"""
    except ValueError:
            print("---------------- Value Error default")
            EXTRACTION_P    = "I" 
            dia_op          = datetime.datetime.now(tz=CST)  -datetime.timedelta(days=5)
            DIA_OPERACION   = dia_op.strftime (DATE_FORMAT)
            DIA_OPERACION_F = dia_op2.strftime(DATE_FORMAT)

    # Acceder a xcom desde variable task instance
    task_instance = kwargs['ti']    
    task_instance.xcom_push(key="EXTRACTION_P", value=EXTRACTION_P)
    task_instance.xcom_push(key="DELTA_DIAS", value=DELTA_DIAS)
    task_instance.xcom_push(key="DIA_OPERACION", value=str(DIA_OPERACION))
    task_instance.xcom_push(key="DIA_OPERACION_F", value=str(DIA_OPERACION_F))
    print(f"-----------Params extraction :{EXTRACTION_P},exec_date:{DIA_OPERACION},exec_date2:{DIA_OPERACION_F}")
    time.sleep(5)


from airflow.utils.dates import days_ago
from airflow.providers.google.cloud.operators.dataflow import DataflowTemplatedJobStartOperator
from airflow.hooks.base_hook import BaseHook
from data_code.airflow.general import conf, conn_account, project_id, snetwork
import gcsfs
import json
import ast

def subdag_wrk(parent_dag_name, child_dag_name, args, json_gs, **kwargs):
    
    connection_airflow_yas_sa_sii_de = BaseHook.get_connection(conn_account)
    service_account_yas_sa_sii_de = ast.literal_eval(connection_airflow_yas_sa_sii_de.extra_dejson["keyfile_dict"])
    
    # Datos Generales para la ejecucion
    project        = conf["proy_sii"]
    temp_location  = "gs://{}/CTA/".format(conf['bucket_tmp'])
    region         = conf["region"]
    service_account_email = conf["service_account_sii"]
    subnetwork     = conf["snetwork"]
    machine_type   = conf["machine_type"]
    with gcsfs.GCSFileSystem(project=project_id, token=service_account_yas_sa_sii_de).open(json_gs) as f:
            jd = json.load(f)
            app= jd['app']
    # Variables para ejecucion desde JSON
    url_trn =  f'gs://{conf["bucket_trn"]}/{jd["app"]}/trn_{jd["table_name"]}/'
    
    # Datos de bq
    job_name_bq = jd['job_name']
    table_id = f"{project_id}:{conf['dataset_r']}.wrk_{jd['table_name']}"
    template_location_bq = f"gs://{conf['bucket_des']}/{jd['app']}/WRK/tpl/{jd['wrk_template_name']}"
    schema_source_bq = f"{conf['dataset_r']}.{jd['table_name']}"  
    delete_columns = jd['delete_columns'] 
    num_workers    = jd['num_workers']
    max_num_workers= jd['max_num_workers']    
    
    folders = gcsfs.GCSFileSystem(project=project, token=service_account_yas_sa_sii_de).ls(url_trn)
    folders.sort()
    folder = folders[-1]    
    date_folder = folder.split('/')[3]
    
    if len(date_folder)==10:
        url_source = 'gs://'+folder+'/'
        
        parent_dag_name_for_id = parent_dag_name.lower()
        
        print('url_source: ' + url_source)
        print('table_dest: ' + table_id+ ' delete: '+delete_columns )
        
        dataflow = DataflowTemplatedJobStartOperator(
            task_id=f'{parent_dag_name_for_id}-{child_dag_name}',
            job_name=f'{job_name_bq}-{date_folder}',
            template=template_location_bq,
            #project_id=project,
            location="us-east1",
            gcp_conn_id=conn_account,
            parameters={ 
                    'url_trn' : url_source,
                    'table_id' : table_id,
                    # 'schema_source_bq' : schema_source_bq,
                    # 'delete_columns' : f"{delete_columns}",
                    },
            default_args=args,
            options={
                "subnetwork":snetwork,
                "serviceAccountEmail":service_account_email,
            },
            dataflow_default_options={
                    'project'      : project,
                    'zone'         : conf['zone'],
                    'tempLocation' : temp_location,
                    'machineType'  : machine_type,
                    'serviceAccountEmail': service_account_email,
                    'subnetwork'   : snetwork,  
                    'ipConfiguration': 'WORKER_IP_PRIVATE',
                },
                # dag=dag,
            )
        dataflow.execute(context=kwargs)

